@@ -12,10 +12,8 @@ test_that("communities are retrieved",{
   zenodo <- ZenodoManager$new(logger = "INFO")
   zen_communities_df <- zenodo$getCommunities()
   expect_is(zen_communities_df, "data.frame")
-  expect_equal(nrow(zen_communities_df), 4515)
   zen_communities_raw <- zenodo$getCommunities(pretty = FALSE)
   expect_is(zen_communities_raw, "list")
-  expect_equal(length(zen_communities_raw),  4515)
 })
 
 test_that("community is retrieved by id",{
